/*
 * ---------------------------------------------
 *               RoboRover James                
 *           με το Arduino Uno Rev. 3           
 * ---------------------------------------------
 *
 *   13ο Μαθητικό Φεστιβάλ Ψηφιακής Δημιουργίας
 *              18-19 Απριλίου 2024
 *           Γυμνάσιο Πυργετού Λάρισας
 *
 *         κώδικας από τον Γιώργο Ρούσσα
 */

#include <Servo.h>

int echoPin=11;   // Pin απάντησης αισθητήρα υπερήχων
int trigPin=12;   // Pin σκανδαλισμού αισθητήρα υπερήχων
int LA=60;        // PWM χαμηλή στάθμη αριστερού κινητήρα
int LB=90;        // PWM μεσαία στάθμη αριστερού κινητήρα
int LC=120;       // PWM υψηλή στάθμη αριστερού κινητήρα
int RA=77;        // PWM χαμηλή στάθμη δεξιού κινητήρα
int RB=113;       // PWM μεσαία στάθμη δεξιού κινητήρα
int RC=141;       // PWM υψηλή στάθμη δεξιού κινητήρα
int HookSrv=10;
boolean USonic = false;  // Ultrasonic Sensor OFF
String readString;
Servo myservo;
int servoAngle=5;
 
/*
 * Αρχικοποίηση
 */
void setup () {
  Serial.begin(9600);

  pinMode(2, OUTPUT);
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(7, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(trigPin, OUTPUT);

  myservo.attach(HookSrv, 500, 2500);
  myservo.write(servoAngle);
}

/*
 * Διαδικασία λήψης και εκτέλεσης εντολής
 */ 
void loop () {
  int lval, rval;
  boolean check=true;
  
  // Ανάγνωση ακολουθίας εντολής από σειριακή είσοδο
  // π.χ. Κονσόλα ή Bluetooth
  //   +++++ Ίσως πρέπει να προστεθεί
  //   if (Serial.available()) readString=""; 
  while (Serial.available()) {
    delay(3); 
    char c = Serial.read();
    readString += c;
  }

  // Απόρριψη υπερσυγκέντρωσης εντολών
  if(readString.length()>2) {
    readString="";
  }
  
  if (readString.length()>=1 && readString.length()<=2 
                             && readString!="-1") {
    // Αποδεκτός συνδυασμός εντολής
    byte c1=readString[0];
    byte c2=readString[1];

    int val= (int)(c1-48) * 10+(int)(c2-48);
    
    // Μετατροπή εντολής σε συνδυασμό τιμών εξόδου κινητήρων
    // και χρήσης αισθητήρα υπερήχων μόνο σε εμπροσθοπορεία 
    //                 ----------------
    //                  40 41 42 43 44
    //                  30 31 32 33 34
    //                  20 21 22 23 24
    //                  10 11 12 13 14
    //                  00 01 02 03 04
    //                 ----------------
    switch (val) {
      case 00:  lval=-LA;  rval=RA;   check=false;  break;
      case 01:  lval=-LA;  rval=RA;   check=false;  break;
      case 02:  lval=-LA;  rval=-RA;  check=false;  break;
      case 03:  lval=LA;   rval=-RA;  check=false;  break;
      case 04:  lval=LA;   rval=-RA;  check=false;  break;
      
      case 10:  lval=0;    rval=RA;   check=false;  break;
      case 11:  lval=0;    rval=RA;   check=false;  break;
      case 12:  lval=0;    rval=0;    check=false;  break;
      case 13:  lval=LA;   rval=0;    check=false;  break;
      case 14:  lval=LA;   rval=0;    check=false;  break;

      case 20:  lval=0 ;   rval=RA;   check=false;  break;
      case 21:  lval=0 ;   rval=RA;   check=false;  break;
      case 22:  lval=LA ;  rval=RA;   check=true;   break;
      case 23:  lval=LA;   rval=0;    check=false;  break;
      case 24:  lval=LA;   rval=0;    check=false;  break;
      
      case 30:  lval=LA ;  rval=RB;   check=true;   break;
      case 31:  lval=LA ;  rval=RB;   check=true;   break;
      case 32:  lval=LB ;  rval=RB;   check=true;   break;
      case 33:  lval=LB;   rval=RA;   check=true;   break;
      case 34:  lval=LB;   rval=RA;   check=true;   break;
      
      case 40:  lval=LA ;  rval=RC;   check=true;   break;
      case 41:  lval=LB ;  rval=RC;   check=true;   break;
      case 42:  lval=LC ;  rval=RC;   check=true;   break;
      case 43:  lval=LC;   rval=RB;   check=true;   break;
      case 44:  lval=LC;   rval=RA;   check=true;   break;

      case 51:  lval=0 ;  rval=0;   check=false;   servoAngle = ((servoAngle) >  5 ? servoAngle-5 : servoAngle);  break;
      case 52:  lval=0 ;  rval=0;   check=false;   servoAngle = ((servoAngle) < 75 ? servoAngle+5 : servoAngle);  break;
      
      case 90:  USonic = false;  lval=0; rval=0;    break;
      case 91:  USonic = true;   lval=0; rval=0;    break;
      case 99:  dance();         lval=0; rval=0;    break;
      
      default:  lval=0;    rval=0;    check=false;  break;
    }
    
    // Επενέργεια Ultrasonic check
    if (USonic == false) check = false;
    
    // Γίνεται έλεγχος για ύπαρξη εμποδίου με υπερήχους
    if (check && distance() < 50) {    // 50 cm περιθώριο
       motion (0, 0);
    }
    else {
      myservo.write(servoAngle);
      motion (lval, rval);
    }
    
    delay(100);
    readString="";
  }  
}

/*
 * Υπολογισμός απόστασης μέσω αισθητήρα υπερήχων
 */
int distance() {
  long duration;
  int dist;
  
  // Καθαρισμός ακίδας σκανδαλισμού
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  
  // Σκανδαλισμός υπερήχων
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  
  // Επιστροφή παλμού αντήχησης
  duration = pulseIn(echoPin, HIGH);

  // Υπολογισμός απόστασης
  dist= (int) duration*0.034/2;
  return dist;
}

/*
 * Εκτέλεση εντολών κινητήρων
 */
void motion(int L_Vel, int R_Vel) {
  
  // Ορισμός ταχύτητας μέσω PWM
  analogWrite (3, abs(L_Vel));
  analogWrite (5, abs(R_Vel));
  delay(3);
  
  // Έλεγχος Αριστερού κινητήρα
  if (L_Vel>0) {
    digitalWrite (2, LOW);
    digitalWrite (4, HIGH);
  }
  else if (L_Vel<0) {
    digitalWrite (2, HIGH);
    digitalWrite (4, LOW);
  }
  else {
    digitalWrite (2, LOW);
    digitalWrite (4, LOW);
  }
  
  // Έλεγχος Δεξιού κινητήρα
  if (R_Vel>0) {
    digitalWrite (6, LOW);
    digitalWrite (7, HIGH);
  }
  else if (R_Vel<0) {
    digitalWrite (6, HIGH);
    digitalWrite (7, LOW);
  }
  else {
    digitalWrite (6, LOW);
    digitalWrite (7, LOW);
  }
}


void dance()
{
  motion (100,120);
  delay(600);
  for(int i=60;i<230;i++) {
    motion(i, -i);
    delay(10);
  }
  motion (100,120);
  delay(600);
  for(int i=60;i<230;i++) {
    motion(-i, i);
    delay(10);
  }
  motion (-130,-80);
  delay(600);
  motion (80,130);
  delay(700);

  for(int i=1;i<=3;i++) { 
    motion(0,180);
    delay(1200);
    motion(150,0);
    delay(1200);
  }

}
